/****************************************************
AboutSQLServer.com blog
Written by Dmitri Korotkevitch

CLR vs. T-SQL: Performance considerations
2013-07-23
*****************************************************/
set nocount on
go

sp_configure 'show advanced options', 1;
reconfigure;
GO
sp_configure 'clr enabled', 1;
reconfigure;
GO
sp_configure 'show advanced options', 0;
reconfigure;
go

if object_id(N'[dbo].[EvenNumberCLR]') is not null 
	drop function [dbo].[EvenNumberCLR]
go

if object_id(N'[dbo].[EvenNumberCLRWithDataAccess]') is not null 
	drop function [dbo].EvenNumberCLRWithDataAccess
go

if object_id(N'[dbo].[EvenNumber]') is not null 
	drop function [dbo].[EvenNumber]
go

if object_id(N'[dbo].[EvenNumberInline]') is not null 
	drop function [dbo].EvenNumberInline
go

if object_id(N'[dbo].[CalcDistance]') is not null 
	drop function [dbo].CalcDistance
go

if object_id(N'[dbo].[CalcDistanceCLR]') is not null 
	drop function [dbo].CalcDistanceCLR
go

if object_id(N'[dbo].[CalcDistanceInline]') is not null 
	drop function [dbo].CalcDistanceInline
go

if exists ( select	*
			from	sys.procedures p
					join sys.schemas s
					on p.schema_id = s.schema_id
			where	s.name = 'dbo'
					and p.name = 'ExistInInterval' ) 
	drop proc dbo.ExistInInterval
go

if exists ( select	*
			from	sys.procedures p
					join sys.schemas s
					on p.schema_id = s.schema_id
			where	s.name = 'dbo'
					and p.name = 'ExistInIntervalCursor' ) 
	drop proc dbo.ExistInIntervalCursor
go

if object_id(N'[dbo].[Concatenate]') is not null 
	drop aggregate dbo.Concatenate
go

if exists ( select	*
			from	sys.procedures p
					join sys.schemas s
					on p.schema_id = s.schema_id
			where	s.name = 'dbo'
					and p.name = 'ExistInIntervalCLR' ) 
	drop proc dbo.ExistInIntervalCLR
go

if exists ( select	*
			from	sys.procedures p
					join sys.schemas s
					on p.schema_id = s.schema_id
			where	s.name = 'dbo'
					and p.name = 'ExistInIntervalReaderCLR' ) 
	drop proc dbo.ExistInIntervalReaderCLR
go


create assembly [CLRTest]
authorization [dbo]
from 'c:\SQL\CLRTest\CLRTest.dll'
with permission_set = safe
GO

create function [dbo].[EvenNumberCLR] (@id int)
returns int
as external name 
	[CLRTest].[UserDefinedFunctions].[EvenNumberCLR];
go

create function [dbo].[EvenNumberCLRWithDataAccess] (@id int)
returns int
as external name 
	[CLRTest].[UserDefinedFunctions].[EvenNumberCLRWithDataAccess];
go

create function [dbo].[CalcDistanceCLR]
(
 @fromLat float(53),
 @fromLon float(53),
 @toLat float(53),
 @toLon float(53)
)
returns float(53)
as external name 
	[CLRTest].[UserDefinedFunctions].[CalcDistanceCLR];
go

create procedure [dbo].[ExistInIntervalCLR]
	@minId int,
	@maxId int,
	@rowCnt int output
as external name 
	[CLRTest].[StoredProcedures].[ExistInIntervalCLR]
go

create procedure [dbo].[ExistInIntervalReaderCLR]
	@minId int,
	@maxId int,
	@rowCnt int output
as external name 
	[CLRTest].[StoredProcedures].[ExistInIntervalReaderCLR]
go

create aggregate [dbo].[Concatenate](@value nvarchar (4000))
returns nvarchar (max)
external name [CLRTest].[Concatenate]
go


if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'Numbers'
)
	drop table dbo.Numbers
go

create table dbo.Numbers
(
	Num int not null,

	constraint PK_Numbers
	primary key clustered(Num)
)
go

-- 262,144 rows
;with N1(C) as (select 0 union all select 0) 
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) 
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) 
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) 
,N5(C) as (select 0 from N4 as T1 cross join N4 as T2)
,N6(C) as (select 0 from N4 as T1 
	cross join N4 as T2 cross join N2 AS T3)
,Nums(Num) as (
	select row_number() over 
		(order by (select null)) from N6)
insert into dbo.Numbers(Num)
	select Num from Nums
go

create function dbo.EvenNumber(@ID int)
returns int
with schemabinding
as 
begin
	return (case when @ID % 2 = 0 then 1 else 0 end)
end
go

create function dbo.EvenNumberInline(@ID int)
returns table
as
return
(
	select (case when @ID % 2 = 0 then 1 else 0 end) as Result
)
go

set statistics time on
-- CLR UDF - no data access context
select count(*) 
from dbo.Numbers 
where dbo.EvenNumberCLR(Num) = 1

-- CLR UDF - data access context
select count(*) 
from dbo.Numbers 
where dbo.EvenNumberCLRWithDataAccess(Num) = 1

-- TSQL - Scalar UDF
select count(*) 
from dbo.Numbers 
where dbo.EvenNumber(Num) = 1;

-- TSQL - Multi-statement UDF
select count(*) 
from 
	dbo.Numbers n cross apply 
		dbo.EvenNumberInline(n.Num) e
where
	e.Result = 1  

set statistics time off
go 

create function dbo.CalcDistance
(
   @FromLat decimal(9,6)
   ,@FromLon decimal(9,6)
   ,@ToLat decimal(9,6)
   ,@ToLon decimal(9,6)
)
returns float
as 
begin
   declare	
      @Dist float
      ,@FromLatR float = radians(@FromLat)
      ,@FromLonR float = radians(@FromLon)
      ,@ToLatR float = radians(@ToLat)
      ,@ToLonR float = radians(@ToLon)

   set @Dist = 
      2 * asin(
         sqrt(
	        power(sin((@FromLatR - @ToLatR) / 2.), 2) + 
            (
               cos(@FromLatR) * 
               cos(@ToLatR) * 
			   power(sin((@FromLonR - @ToLonR) / 2.0), 2)
            )
         )
	  ) * 20001600. / pi()
	
	return @Dist
end
go

create function dbo.CalcDistanceInline
(
   @FromLat decimal(9,6)
   ,@FromLon decimal(9,6)
   ,@ToLat decimal(9,6)
   ,@ToLon decimal(9,6)
)
returns table
as
return
(
   with	Rads(FromLatR, FromLonR, ToLatR, ToLonR)
   as 
   (
      select 
	     radians(@FromLat), radians(@FromLon),
         radians(@ToLat), radians(@ToLon)
   )
   select	
     2 * asin(
	    sqrt(
		   power(sin((FromLatR - ToLatR) / 2.), 2) + 
		   (
		      cos(FromLatR) * 
              cos(ToLatR) * 
              power(sin((FromLonR - ToLonR) / 2.0),2)
            )
		)
	 ) * 20001600. / pi() as Distance
   from Rads
)
go

set statistics time on

select count(*)
from dbo.Numbers
where dbo.CalcDistance(
	Num % 89, Num % 179, 
	Num % 89 + 1, Num % 179 + 1) > 0

select count(*)
from dbo.Numbers
where dbo.CalcDistanceClr(
	Num % 89, Num % 179, 
	Num % 89 + 1, Num % 179 + 1) > 0


select count(*)
from 
	dbo.Numbers n cross apply
		dbo.CalcDistanceInline
			(n.Num % 89, n.Num % 179, 
			n.Num % 89 + 1, n.Num % 179 + 1) d
where	
	d.Distance > 0

set statistics time off
go


create proc dbo.ExistInInterval
(
 @MinId int,
 @MaxId int,
 @RowCount int output
)
as 
set nocount on
	
set @RowCount = 0
while @MinId <= @MaxId 
	begin
		if exists ( select	*
					from	dbo.Numbers
					where	Num = @MinId ) 
			set @RowCount += 1
		set @MinId += 1
	end
go

declare	
	@rowCnt int
	,@dt datetime = getdate()
exec dbo.ExistInInterval @MinId = 0, @MaxId = 50000, @RowCount  = @RowCnt output
select @RowCnt,datediff(millisecond,@dt,getdate())
select @dt = getdate()
exec dbo.ExistInIntervalClr @MinId = 0, @MaxId = 250000, @RowCnt = @RowCnt output
select @RowCnt,datediff(millisecond,@dt,getdate())
go

create proc dbo.ExistInIntervalCursor
(
   @MinId int
   ,@MaxId int
   ,@RowCount int output
)
as 
   set nocount on
   declare 
      @Num int  

   declare 
      curWork cursor fast_forward
      for
         select Num
         from dbo.Numbers
         where Num between @MinId and @MaxId

   set @RowCount = 0
   
   open curWork
   fetch next from curWork into @Num	
   while @@fetch_status = 0 
   begin
      set @RowCount += 1
      fetch next from curWork into @Num	
   end  
   close curWork
   deallocate curWork
go

declare	
	@rowCnt int
	,@dt datetime = getdate()
exec dbo.ExistInIntervalCursor @MinId = 0, @MaxId = 250000, @RowCount  = @RowCnt output
select @RowCnt,datediff(millisecond,@dt,getdate())
select @dt = getdate()
exec dbo.ExistInIntervalReaderClr @MinId = 0, @MaxId = 250000, @RowCnt = @RowCnt output
select @RowCnt,datediff(millisecond,@dt,getdate())
go 


declare	
   @V nvarchar(max) = N''
	,@MaxNum int = 100000
	,@DT datetime = getdate()
select @V = @V + convert(nvarchar(32), Num) + ','
from dbo.Numbers
where Num <= @MaxNum

-- removing trailing comma
select @V = case when @V = '' then '' else left(@V,len(@V) - 1) end
select datediff(millisecond,@dt,getdate())
select @DT = getdate()

select len(case when Result is null then '' else left(Result,len(Result) - 1) end)
from
   (
      select 
         convert(nvarchar(max), 
            (
               select Num as [text()], ',' as [text()]
               from dbo.Numbers
               where Num <= @MaxNum
               for xml path('')
            )
         ) as Result
   ) r

select datediff(millisecond,@dt,getdate())
select @DT = getdate()

select	len(dbo.Concatenate(Num))
from	dbo.Numbers
where	Num <= @MaxNum
select datediff(millisecond,@dt,getdate())
go
